@@ -39,7 +39,7 @@ class diamond:
         certificates: List[str],
         store: store,
         id: int
-    ) -> None: #all attributes are made private
+    ) -> None:
         self._carat = carat
         self._cut = cut
         self._color = color
@@ -101,7 +101,8 @@ class diamond:
 
 #--------------------------------------------FUNCTIONS-----------------------------------------------
 
-#Attempt connection with provided configuration. If successful, store configuration in config.txt file.
+#ATTEMPT CONNECTION WITH PROVIDED CONFIGURATION.
+#IF SUCCESSFUL, STORE CONFIGURATION IN CONFIG.TXT FILE.
 def configure(
     rdb_host: str,
     rdb_port: int,
@@ -133,14 +134,13 @@ def configure(
         cursor = connection.cursor()
 
         #Check tables exist and if not, create them
-
         if _tableNotExist(cursor,rdb_database,'stores'):#checking and creating stores table
             sqlCommand = """CREATE TABLE stores (
                 ID INT(11) NOT NULL AUTO_INCREMENT,
                 city VARCHAR(80) NOT NULL,
                 country VARCHAR(80) NOT NULL,
                 PRIMARY KEY (ID)
-                );""" #even though there is only 1 store per city (and so city could be used as the primary key), there could be 2 cities with the same name in different countries. I could combine city and country to make a primary key, but doing an int ID made things easier and didn't increase the database size by much.
+                );""" #even though there is only 1 store per city (and so city could be used as the primary key), there could be 2 cities with the same name in different countries. I could combine city and country to make a primary key, but doing an ID made things easier and didn't increase the database size by much.
             cursor.execute(sqlCommand)
 
         if _tableNotExist(cursor,rdb_database,'diamonds'):#checking and creating diamonds table
@@ -166,8 +166,8 @@ def configure(
     except connector.Error as e:
         print('Error while connecting to MySQL: ', e)
     
-#Check that the table doesn't already exist
-def _tableNotExist(cursor, db, tableName) -> bool:
+#CHECK THAT THE TABLE DOESN'T ALREADY EXIST
+def _tableNotExist(cursor, db:str, tableName:str) -> bool:
     sqlCommand = f"""SELECT COUNT(*) FROM information_schema.tables 
     WHERE table_schema = '{db}' 
     AND table_name = '{tableName}';""" #count all tables with the inputted name
@@ -180,7 +180,7 @@ def _tableNotExist(cursor, db, tableName) -> bool:
 
 
 
-#Add a new diamond to the registry/database
+#ADD A NEW DIAMOND TO THE REGISTRY/DATABASE
 def add_diamond(
     carat: float,
     cut: str,
@@ -200,6 +200,23 @@ def add_diamond(
     connection = _connectToDatabase()#connect to database
     cursor = connection.cursor()
 
+    #INPUT CHECKS:
+    cut = cut.capitalize()#capitalize first letter, lower case the rest
+    if cut not in ['Fair','Good','Very Good','Premium','Ideal']: #check if valid cut
+        raise ValueError('Invalid cut entered - only Fair, Good, Very Good, Premium or Ideal accepted')
+    color = color.capitalize()#capitalize the letter
+    if color not in ['J','I','H','G','F','E','D']:
+        raise ValueError('Invalid color entered - only J, I, H, G, F, E or D accepted')
+    clarity = clarity.upper()#capitalize the input
+    if clarity not in ['I1','SI2','SI1','VS2','VS1','VVS2','VVS1','IF']:
+        raise ValueError('Invalid clarity entered - only I1, SI2, SI1, VS2, VS1, VVS2, VVS1 or IF accepted')
+    for i in range(len(certificates)):
+        certificates[i]=certificates[i].upper()#capitalize the input
+        if certificates[i] not in ['GIA','AGS','GCAL','IGI','EGL']:
+            raise ValueError('Invalid certificate entered - only GIA, AGS, GCAL, IGI or EGL accepted')
+    store_city=store_city.capitalize()#capitalize first letter, lower case the rest
+    store_country=store_country.capitalize()#capitalize first letter, lower case the rest
+
     currentStore = store(store_city,store_country)#create a store with given info
     diamondToAdd = diamond(carat,cut,color,clarity,depth,table,price,x,y,z,certificates,currentStore, id)#create a diamond with given info
     
@@ -222,8 +239,8 @@ def add_diamond(
         connection.close()
     return id
 
-#Add given diamond to the database
-def _addDiamondToDatabase(cursor, diamond, id=None):
+#ADD GIVEN DIAMOND TO THE DATABASE
+def _addDiamondToDatabase(cursor, diamond:diamond, id=None):
     storeId = _getStoreID(cursor,diamond.getStore().getCity(),diamond.getStore().getCountry())#retrieve the ID of the store from the database
     
     if storeId == None:#if the store doesn't exist in the database, add it
@@ -238,7 +255,7 @@ def _addDiamondToDatabase(cursor, diamond, id=None):
         except connector.Error as e:
             print('Error while connecting to MySQL: ', e)
 
-    if id==None:#If no ID was specified
+    if id==None:#If no diamond ID was specified
         sqlCommand = f"""INSERT INTO diamonds (price, carat, cut, color, clarity, x, y, z, depth, table_, certificates, storeID)
         VALUES ({diamond.getPrice()},
         {diamond.getCarat()},
@@ -252,7 +269,7 @@ def _addDiamondToDatabase(cursor, diamond, id=None):
         {diamond.getTable()},
         '{diamond.getCertificates()}',
         '{storeId[0]}');"""#insert diamond into the database with an autogenerated ID
-    else:#If an ID was specified
+    else:#If a diamond ID was specified
         sqlCommand = f"""INSERT INTO diamonds
             VALUES ({diamond.getId()},
             {diamond.getPrice()},
@@ -272,8 +289,8 @@ def _addDiamondToDatabase(cursor, diamond, id=None):
     except connector.Error as e:
             print('Error while connecting to MySQL: ', e)
 
-#Checks that the given diamond ID number doesn't already exist in the database
-def _isIdUnique(cursor,id) -> bool:
+#CHECKS THAT THE GIVEN DIAMOND ID NUMBER DOESN'T ALREADY EXIST IN THE DATABASE
+def _isIdUnique(cursor,id:int) -> bool:
     sqlCommand = f"SELECT COUNT(*) FROM diamonds WHERE ID={id}"#count all the entries with the given ID number
     try:
         cursor.execute(sqlCommand)
@@ -282,8 +299,8 @@ def _isIdUnique(cursor,id) -> bool:
         print('Error while connecting to MySQL: ', e)
     return result[0]==0 #if the count os 0, return True
 
-#Gets the ID that matches the given store info
-def _getStoreID(cursor,city,country):
+#GETS THE ID THAT MATCHES THE GIVEN STORE INFO
+def _getStoreID(cursor, city:str, country:str):
     sqlCommand = f"""SELECT ID FROM stores WHERE 
     city='{_fixApostrophe(city)}' AND 
     country='{_fixApostrophe(country)}';"""
@@ -295,17 +312,17 @@ def _getStoreID(cursor,city,country):
         print('Error while connecting to MySQL: ', e)
     return result
 
-#Connect to the configured database and return the connection
+#CONNECT TO THE CONFIGURED DATABASE AND RETURN THE CONNECTION
 def _connectToDatabase() -> connector:
     #Get configuration data from file
     file = open('config.txt','r') 
     lines = file.readlines()
     file.close()
 
-    rdb_host = lines[0].split(': ')[1]
-    rdb_port = lines[1].split(': ')[1]#port information nt neccessary for this connection
-    rdb_database = lines[2].split(': ')[1]
-    rdb_username = lines[3].split(': ')[1]
+    rdb_host = lines[0].split(': ')[1].rstrip()
+    rdb_port = lines[1].split(': ')[1]#port information not neccessary for this connection
+    rdb_database = lines[2].split(': ')[1].rstrip()
+    rdb_username = lines[3].split(': ')[1].rstrip()
     rdb_password = lines[4].split(': ')[1].rstrip()
 
     #Attempt connection
@@ -316,22 +333,58 @@ def _connectToDatabase() -> connector:
                 user=rdb_username,
                 password=rdb_password)
 
+        cursor = connection.cursor()
+        #Check if tables exist and if not, create them
+        if _tableNotExist(cursor,rdb_database,'stores'):#checking and creating stores table
+            sqlCommand = """CREATE TABLE stores (
+                ID INT(11) NOT NULL AUTO_INCREMENT,
+                city VARCHAR(80) NOT NULL,
+                country VARCHAR(80) NOT NULL,
+                PRIMARY KEY (ID)
+                );"""
+            cursor.execute(sqlCommand)
+
+        if _tableNotExist(cursor,rdb_database,'diamonds'):#checking and creating diamonds table
+            sqlCommand = """CREATE TABLE diamonds (
+                ID INT(11) NOT NULL AUTO_INCREMENT,
+                price SMALLINT,
+                carat FLOAT(3,2) NOT NULL,
+                cut ENUM('Fair','Good','Very Good','Premium','Ideal') NOT NULL,
+                color ENUM('J','I','H','G','F','E','D') NOT NULL,
+                clarity ENUM('I1','SI2','SI1','VS2','VS1','VVS2','VVS1','IF') NOT NULL,
+                x FLOAT(4,2) NOT NULL,
+                y FLOAT(4,2) NOT NULL,
+                z FLOAT(4,2) NOT NULL,
+                depth FLOAT(4,1) NOT NULL,
+                table_ FLOAT(4,1) NOT NULL,
+                certificates SET('GIA','AGS','GCAL','IGI','EGL'),
+                storeID INT(11),
+                PRIMARY KEY (ID),
+                FOREIGN KEY (storeID) REFERENCES stores(ID)
+            );"""
+            cursor.execute(sqlCommand)
+
         return connection
     except connector.Error as e:
         print('Error while connecting to MySQL: ', e)
 
 
-#Gets all matching diamonds from the database.
+#GETS ALL MATCHING DIAMONDS FROM THE DATABASE.
 # If only country is provided, matching diamonds from all stores within that country is returned.
 def get_diamonds(
     store_city: Optional[str] = None,
     store_country: Optional[str] = None,
     required_certificates: Optional[List[str]] = None,
 ) -> List[Any]:
-    connection = _connectToDatabase()
+    connection = _connectToDatabase()#connect to the database
     cursor = connection.cursor()
 
-    if (store_city == None and store_country == None): #if no store was given of found in the database
+    if store_city!=None: store_city=store_city.capitalize()#capitalize first letter, lower case the rest
+    if store_country!=None: store_country=store_country.capitalize()#capitalize first letter, lower case the rest
+    for i in range(len(required_certificates)):
+        required_certificates[i]=required_certificates[i].upper()#capitalize the input
+
+    if (store_city == None and store_country == None): #if no store was given or found in the database
         if required_certificates == None: #and no certificates given
             sqlCommand = """SELECT * FROM diamonds;""" #get all diamonds from the database
         else:#if only certificates were provided
@@ -347,7 +400,7 @@ def get_diamonds(
         for store in storeIDs: #loop through all the storeID options
             sqlCommand+=f" storeID={store[0]} OR"
         sqlCommand = sqlCommand[:-2] + ")"
-        if required_certificates != None:
+        if required_certificates != None:#if certificates were also provided
             sqlCommand+=" AND "
             for cert in required_certificates:
                 sqlCommand += f" certificates LIKE '%{cert}%' AND"
@@ -356,8 +409,10 @@ def get_diamonds(
     
     cursor.execute(sqlCommand)
     results = cursor.fetchall()
-    fields = ['id','diamond'] #set up callable lables, 'id' was requested as a lable
+
+    fields = ['id','diamond'] #set up lables, 'id' was requested as a lable
     allDiamonds = []
+
     for entry in results:#for each result, store the diamond object and it's id in a dictionary format
         (id,price,carat,cut,color,clarity,x,y,z,depth,table,certificates,storeID) = entry
         currentDiamond = diamond(carat,cut,color,clarity,depth,table,price,x,y,z,certificates,_getStore(cursor,storeID),id)
@@ -367,8 +422,8 @@ def get_diamonds(
 
     return allDiamonds
 
-#return storeIDs for all stored that match the city/country info provided
-def _getAllMatchingStoreIDs(cursor,city,country) -> List[int]:
+#RETURN STOREIDS FOR ALL STORES THAT MATCH THE CITY/COUNTRY INFO PROVIDED
+def _getAllMatchingStoreIDs(cursor, city : str, country : str) -> List[int]:
     sqlCommand = """SELECT ID FROM stores WHERE """
     if city != None:
         sqlCommand += f"city='{_fixApostrophe(city)}'"
@@ -385,8 +440,8 @@ def _getAllMatchingStoreIDs(cursor,city,country) -> List[int]:
         print('Error while connecting to MySQL: ', e)
     return results
 
-#return store object from given store ID
-def _getStore(cursor, id) -> store:
+#RETURN STORE OBJECT FROM GIVEN STORE ID
+def _getStore(cursor, id : int) -> store:
     sqlCommand = f"""SELECT city, country FROM stores WHERE ID={id}"""
     try:
         cursor.execute(sqlCommand)
@@ -396,7 +451,7 @@ def _getStore(cursor, id) -> store:
     return store(city,country)
 
 
-#Deletes all tables and data from the system
+#DELETES ALL TABLES AND DATA FROM THE SYSTEM
 def delete_all() -> None: 
     connection = _connectToDatabase()
     cursor = connection.cursor()
@@ -410,7 +465,7 @@ def delete_all() -> None:
     connection.close()
 
 
-#Import data from remote URL
+#IMPORT DATA FROM REMOTE URL
 def batch_import(
     url: str
 ) -> None:
@@ -423,17 +478,19 @@ def batch_import(
     cursor = connection.cursor()
     for row in cr:#for each row in the file
         certificates = row[11].split(',')#split the line up into data pieces
-        currentStore = store(row[12],row[13])
-        diamondToAdd = diamond(float(row[1]),row[2],row[3],row[4],float(row[5]),float(row[6]),int(row[7]),float(row[8]),float(row[9]),float(row[10]),certificates,currentStore,int(row[0]))
+        currentStore = store(row[12],row[13])#create a store object
+        diamondToAdd = diamond(float(row[1]),row[2],row[3],row[4],float(row[5]),float(row[6]),int(row[7]),float(row[8]),float(row[9]),float(row[10]),certificates,currentStore,int(row[0]))#create a diamond object
         _addDiamondToDatabase(cursor, diamondToAdd)#add diamond to database
         connection.commit()
     connection.close()
 
-#Changes single apostrophes into double apostrophes to abide with SQL text format (and prevent sql injection)
+#CHANGES SINGLE APOSTROPHES INTO DOUBLE APOSTROPHES TO ABIDE WITH SQL TEXT FORMAT (AND PREVENT SQL INJECTION)
 def _fixApostrophe(word: str) -> str:
     if "'" in word:#if there is at least one apostrophy
         parts = word.split("'")#split on the apostrophies
         word = parts[0]
         for section in parts[1:]:
-            word = word+"''"+section#and rejoin with
+            word = word+"''"+section#and rejoin with double apostrophies instead of a single apostrophy
     return word
+
+print(get_diamonds(store_city="Tol'yatti", required_certificates=['IGI','GIA'])[0]['id'])
\ No newline at end of file
